new;
library tsm,optmum;
TSMset;

/* Simulate an arma(1,1) process */

{data1,retcode} = RND_arfima(0.5|-0.2|0,1,1,1,1000,512,1);

data2 = (data1 + cos(data1) )^2;

_fourier = 0;  /* Using the fast transform */
_surrogate_random = 1; /* uniform [0,2pi] randomization of the phases */

output file = nonlin.out reset;

seed = 123;
call Test(data1,25,3,4,seed);

print;

seed = 123;
call Test(data2,25,3,4,seed);

output off;


/* {sigma,pvalue} = Test(y,Ns,order1,order2,seed} */

/* y: data                                      */
/* Ns: number of FT surrogate simulations       */
/* order1:  the linear autoregressive order     */
/* order2:  the non-linear autoregressive order */
/* seed: seed parameter                         */

/* sigma: the surrogate test                    */
/* pvalue: p-value of the surrogate test        */


proc (2) = Test(y,Ns,order1,order2,seed);
  local simul,TR2,pvalue,TR2s,s,ys,sigma;

  y = packr(y);

  if cols(y) /= 1;
    ERRORLOG "error: Not valid for multivariate data.";
    end;
  endif;

  {TR2,pvalue} = LWG(y,order1,order2,seed);

  simul = {};

  s = 1;
  do until s > Ns;
    cls; locate 4,5; printdos ftos(s,"surrogate N%lf",5,0);
    ys = surrogate(y);
    simul = simul~ys;
    s = s + 1;
  endo;

  TR2s = zeros(Ns,1);

  s = 1;
  do until s > Ns;
    {TR2s[s],pvalue} = LWG(simul[.,s],order1,order2,seed);
    s = s + 1;
  endo;

  sigma = abs(TR2 - meanc(TR2s))/stdc(TR2s);
  pvalue = erfc(sigma/sqrt(2));

  if _print == 1;

    cls;
    print "----------------------------------------------";
    print "LWG-Surrogate test for neglected non-linearity";
    print "----------------------------------------------";
    print;
    print ftos(Ns,"Number of surrogate:                 %lf",1,0);
    print;
    print ftos(TR2,"LWG test for the original data:      %lf",5,5);
    print ftos(meanc(TR2s),"LWG test for the surrogate data:      %lf",5,5);
    print;
    print ftos(sigma,"Surrogate test:                       %lf",5,5);
    print ftos(pvalue,"p_value:                              %lf",5,5);

  endif;

  retp(sigma,pvalue);
endp;


proc (2) = LWG(data,p,J,seed);
  local tmin,tmax,t,Nobs,y,x,i,xi,z;
  local K,beta,u,phi,M,delta,e,R2,TR2,pvalue;

  /* target data */

  tmin = 0.1; tmax = 0.9;
  t = tmin + (tmax-tmin)*(data-minc(data)')./(maxc(data)'-minc(data)');

  Nobs = rows(t);

  /* endogeneous data */

  y = t[p+1:Nobs];

  /* exogeneous data */

  x = {};
  i = 1;
  do until i > p;
    xi = t[1+p-i:Nobs-i];
    x = x~xi;
    i = i+1;
  endo;

  /* Add a constant */

  z = ones(Nobs-p,1)~x;

  I = cols(z);    /* Input Number */
  K = cols(y);    /* Output Number */

  /* OLS: data(t) = beta0 + beta1*data(t-1) + ... + betap*data(t-p) + u(t) */

  beta = y/z;

  /* Residuals */

  u = y - z*beta;

  /* Neural Net */

  phi = Activation(z,J,seed);

  /* Form the regressor matrix */

  M = z~phi;

  /* Residuals of the Neural Net model */

  delta = u/M;
  e = u - M*delta;

  /* T*R^2 */

  R2 = 1 - (e'e)/(u'u);
  TR2 = (Nobs-p)*R2;

  /* pvalue */

  pvalue = cdfchic(TR2,J);

  retp(TR2,pvalue);
endp;


proc Net_Function(x);
  local y;
  y = 1./(1+exp(-x));
  retp(y);
endp;


proc Activation(x,J,seed);
  local Nx,u,ht,at;

  rndseed seed;

  Nx = cols(x);

  u = 4*rndu(J,Nx) - 2;  /* [-2,2] uniform random */
  ht = x*u';
  at = Net_Function(ht);

  retp(at);
endp;
