/*
**  y(t) = b(t)*x(t) + u(t)
**  b(t) = alpha*b(t-1) + delta*z(t) + v(t)
*/

new;
library tsm,optmum,pgraph;
declare external sigma;
declare external delta;
declare external alpha;

rndseed 123456;

Nobs = 100;
s = seqa(1,1,100);
x = rndu(Nobs,1)*10;
z = rndu(Nobs,1);
v = rndn(Nobs,1)*1.5;
b = recserar(v+0.25*z,0,0.8);
u = rndn(Nobs,1)*2;
y = b.*x + u;

proc ZZ(i);
  retp(x[i]);
endp;

proc d(i);
  retp(0);
endp;

proc T(i);
  retp(alpha);
endp;

proc c(i);
  retp(delta*z[i]);
endp;

proc R(i);
  retp(1);
endp;

proc H(i);
  local w;
  w = sigma[1]^2;
  retp(w);
endp;

proc Q(i);
  local w;
  w = sigma[2]^2;
  retp(w);
endp;

a0 = 0; P0 = 0;

proc ml(theta);
  local LogL;
  alpha = theta[1];
  delta = theta[2];
  sigma = theta[3 4];
  call SSM_build(&ZZ,&d,&H,&T,&c,&R,&Q,1);
  call KFiltering(y,a0,P0);
  LogL = KF_ml;
  retp(LogL);
endp;

output file = varying.out reset;

sv = ones(4,1);
_tsm_parnm = "alpha"|"delta"|"sig_u"|"sig_v";
{theta,stderr,Mcov,LogL} = TD_ml(&ml,sv);

output off;

alpha = theta[1]; delta = theta[2]; sigma = theta[3 4];
call SSM_build(&ZZ,&d,&H,&T,&c,&R,&Q,1);
call KFiltering(Y,a0,P0);

yc = KF_matrix(1);     /* y[t|t-1]      */
v = KF_matrix(2);      /* v[t]          */
a = KF_matrix(3);      /* a[t]          */
P = KF_matrix(4);      /* P[t]          */
ac = KF_matrix(5);     /* a[t|t-1]      */
Pc = KF_matrix(6);     /* P[t|t-1]      */
F = KF_matrix(7);      /* F[t|t-1]      */
invF = KF_matrix(8);   /* F[t|t-1]^(-1) */

{as,Ps} = Ksmoothing;

graphset;
  _pdate = ""; _pnum = 2; fonts("simplex simgrma");
  xlabel("Observation");
  _plegstr = "\202b\201\000Kalman filter\000Kalman smoothing";
  _plegctl = {2 6 5 1};
  _pltype = 6|1|3;
  xy(s,b~a~as);



