new;
library tsm,finance,optmum;
TSMset;

rndseed 123;

/* Simulation d'un processus */

x0 = 0.1; a = 0.8; b = 0.1; sigma = 0.06;
h = 0.1;
Nobs = 10000;

k1 = exp(-a*h); k2 = (1-exp(-2*a*h))/(2*a);
data = recserar(b*(1-k1) + (sigma*sqrt(k2))*rndn(Nobs,1),x0,k1);

/* Observations discretes avec h = 1 */

data = data[seqa(1,10,1000)];
h = 1;

/* Estimation Naive */

Nobs = 1000;

proc ml(theta);
  local a,b,sigma,et,vt,logl;

  a = theta[1]; b = theta[2]; sigma = theta[3];

  et = data[2:Nobs] - data[1:Nobs-1] - a*(b-data[1:Nobs-1])*h;
  vt = (sigma^2)*h;

  logl = -0.5*ln(2*pi) - 0.5*ln(vt) - 0.5*et^2./vt;

  retp(logl);
endp;

{theta_ML,stderr,Mcov,Logl} = TD_ml(&ml,0.5|0.5|0.5);


/* Estimation Indirecte */

proc OU_mu(theta,x);
  local a,b;
  a = theta[1];
  b = theta[2];
  retp(a*(b-x));
endp;

proc OU_sigma(theta,x);
  local sigma;
  sigma = theta[3];
  retp(sigma);
endp;

proc (1) = rnd(theta);
  local x0,a,b,sigma,Nobs,h,k1,k2,x;

  x0 = data[1];
  a = theta[1];
  b = theta[2];
  sigma = theta[3];
  Nobs = 10000;
  h = 0.1;

  k1 = exp(-a*h); k2 = (1-exp(-2*a*h))/(2*a);
  x = recserar(b*(1-k1) + (sigma*sqrt(k2))*rndn(Nobs,1),x0,k1);

  x = x[seqa(1,10,1000)];

  retp(x);
endp;

sv = 0.8|0.1|0.06;
theta_EI =
   Inference_Indirecte(&OU_mu,&OU_sigma,1,3,&rnd,123,sv,theta_ML,eye(3));

output file = eds4.out reset;

_tsm_parnm = "a"|"b"|"sigma";

print; print;
print "              Vraies valeurs    Est. Naive   Inference Indirecte";
print;
call printfmt(_tsm_parnm~sv~theta_ML~theta_EI,0~1~1~1);

output off;
